@@ -12,7 +12,7 @@
 ** This is the header file for the generic hash-table implemenation
 ** used in SQLite.
 **
-** $Id: hash.h,v 1.24 2004/08/09 13:08:30 matt Exp $
+** $Id: hash.h,v 1.25 2004/09/10 15:33:00 matt Exp $
 */
 #ifndef _SQLITE_HASH_H_
 #define _SQLITE_HASH_H_
@@ -70,7 +70,7 @@ struct HashElem {
 ** A copy of the key is made for SQLITE_HASH_STRING and SQLITE_HASH_BINARY
 ** if the copyKey parameter to HashInit is 1.  
 */
-#define SQLITE_HASH_INT       1
+/* #define SQLITE_HASH_INT       1 // NOT USED */
 /* #define SQLITE_HASH_POINTER   2 // NOT USED */
 #define SQLITE_HASH_STRING    3
 #define SQLITE_HASH_BINARY    4
@@ -107,6 +107,3 @@ void sqlite3HashClear(Hash*);
 #define sqliteHashCount(H)  ((H)->count)
 
 #endif /* _SQLITE_HASH_H_ */
-
-
-
