@@ -11,7 +11,7 @@
 *************************************************************************
 ** This file contains code used to implement the PRAGMA command.
 **
-** $Id: pragma.c,v 1.8 2004/08/09 13:08:31 matt Exp $
+** $Id: pragma.c,v 1.9 2004/09/10 15:33:01 matt Exp $
 */
 #include "sqliteInt.h"
 #include <ctype.h>
@@ -114,10 +114,13 @@ static int changeTempStorage(Parse *pParse, const char *zStorageType){
 /*
 ** Generate code to return a single integer value.
 */
-static void returnSingleInt(Vdbe *v, const char *zLabel, int value){
+static void returnSingleInt(Parse *pParse, const char *zLabel, int value){
+  Vdbe *v = sqlite3GetVdbe(pParse);
   sqlite3VdbeAddOp(v, OP_Integer, value, 0);
-  sqlite3VdbeSetNumCols(v, 1);
-  sqlite3VdbeSetColName(v, 0, zLabel, P3_STATIC);
+  if( pParse->explain==0 ){
+    sqlite3VdbeSetNumCols(v, 1);
+    sqlite3VdbeSetColName(v, 0, zLabel, P3_STATIC);
+  }
   sqlite3VdbeAddOp(v, OP_Callback, 1, 0);
 }
 
@@ -149,7 +152,8 @@ static int flagPragma(Parse *pParse, const char *zLeft, const char *zRight){
       if( zRight==0 ){
         v = sqlite3GetVdbe(pParse);
         if( v ){
-          returnSingleInt(v, aPragma[i].zName, (db->flags&aPragma[i].mask)!=0);
+          returnSingleInt(pParse,
+               aPragma[i].zName, (db->flags&aPragma[i].mask)!=0);
         }
       }else if( getBoolean(zRight) ){
         db->flags |= aPragma[i].mask;
@@ -277,7 +281,7 @@ void sqlite3Pragma(
     Btree *pBt = pDb->pBt;
     if( !zRight ){
       int size = pBt ? sqlite3BtreeGetPageSize(pBt) : 0;
-      returnSingleInt(v, "page_size", size);
+      returnSingleInt(pParse, "page_size", size);
     }else{
       sqlite3BtreeSetPageSize(pBt, atoi(zRight), 0);
     }
@@ -300,7 +304,7 @@ void sqlite3Pragma(
   if( sqlite3StrICmp(zLeft,"cache_size")==0 ){
     if( sqlite3ReadSchema(pParse) ) goto pragma_out;
     if( !zRight ){
-      returnSingleInt(v, "cache_size", pDb->cache_size);
+      returnSingleInt(pParse, "cache_size", pDb->cache_size);
     }else{
       int size = atoi(zRight);
       if( size<0 ) size = -size;
@@ -322,7 +326,7 @@ void sqlite3Pragma(
   */
   if( sqlite3StrICmp(zLeft, "temp_store")==0 ){
     if( !zRight ){
-      returnSingleInt(v, "temp_store", db->temp_store);
+      returnSingleInt(pParse, "temp_store", db->temp_store);
     }else{
       changeTempStorage(pParse, zRight);
     }
@@ -340,7 +344,7 @@ void sqlite3Pragma(
   if( sqlite3StrICmp(zLeft,"synchronous")==0 ){
     if( sqlite3ReadSchema(pParse) ) goto pragma_out;
     if( !zRight ){
-      returnSingleInt(v, "synchronous", pDb->safety_level-1);
+      returnSingleInt(pParse, "synchronous", pDb->safety_level-1);
     }else{
       if( !db->autoCommit ){
         sqlite3ErrorMsg(pParse, 
@@ -538,7 +542,7 @@ void sqlite3Pragma(
       { OP_MemLoad,     0, 0,        0},
       { OP_Integer,     0, 0,        0},
       { OP_Ne,          0, 0,        0},    /* 2 */
-      { OP_String8,      0, 0,        "ok"},
+      { OP_String8,     0, 0,        "ok"},
       { OP_Callback,    1, 0,        0},
     };
 
@@ -577,7 +581,7 @@ void sqlite3Pragma(
          sqlite3MPrintf("*** in database %s ***\n", db->aDb[i].zName),
          P3_DYNAMIC);
       sqlite3VdbeAddOp(v, OP_Pull, 1, 0);
-      sqlite3VdbeAddOp(v, OP_Concat8, 2, 1);
+      sqlite3VdbeAddOp(v, OP_Concat, 0, 1);
       sqlite3VdbeAddOp(v, OP_Callback, 1, 0);
 
       /* Make sure all the indices are constructed correctly.
@@ -589,15 +593,7 @@ void sqlite3Pragma(
         int loopTop;
 
         if( pTab->pIndex==0 ) continue;
-        sqlite3VdbeAddOp(v, OP_Integer, i, 0);
-        sqlite3VdbeAddOp(v, OP_OpenRead, 1, pTab->tnum);
-        sqlite3VdbeAddOp(v, OP_SetNumColumns, 1, pTab->nCol);
-        for(j=0, pIdx=pTab->pIndex; pIdx; pIdx=pIdx->pNext, j++){
-          if( pIdx->tnum==0 ) continue;
-          sqlite3VdbeAddOp(v, OP_Integer, pIdx->iDb, 0);
-          sqlite3VdbeOp3(v, OP_OpenRead, j+2, pIdx->tnum, 
-                         (char*)&pIdx->keyInfo, P3_KEYINFO);
-        }
+        sqlite3OpenTableAndIndices(pParse, pTab, 1, OP_OpenRead);
         sqlite3VdbeAddOp(v, OP_Integer, 0, 0);
         sqlite3VdbeAddOp(v, OP_MemStore, 1, 1);
         loopTop = sqlite3VdbeAddOp(v, OP_Rewind, 1, 0);
@@ -606,11 +602,11 @@ void sqlite3Pragma(
           int jmp2;
           static VdbeOpList idxErr[] = {
             { OP_MemIncr,     0,  0,  0},
-            { OP_String8,      0,  0,  "rowid "},
+            { OP_String8,     0,  0,  "rowid "},
             { OP_Recno,       1,  0,  0},
-            { OP_String8,      0,  0,  " missing from index "},
-            { OP_String8,      0,  0,  0},    /* 4 */
-            { OP_Concat8,      4,  0,  0},
+            { OP_String8,     0,  0,  " missing from index "},
+            { OP_String8,     0,  0,  0},    /* 4 */
+            { OP_Concat,      2,  0,  0},
             { OP_Callback,    1,  0,  0},
           };
           sqlite3GenerateIndexKey(v, pIdx, 1);
@@ -632,9 +628,9 @@ void sqlite3Pragma(
              { OP_MemLoad,      2,  0,  0},
              { OP_Eq,           0,  0,  0},  /* 7 */
              { OP_MemIncr,      0,  0,  0},
-             { OP_String8,       0,  0,  "wrong # of entries in index "},
-             { OP_String8,       0,  0,  0},  /* 10 */
-             { OP_Concat8,       2,  0,  0},
+             { OP_String8,      0,  0,  "wrong # of entries in index "},
+             { OP_String8,      0,  0,  0},  /* 10 */
+             { OP_Concat,       0,  0,  0},
              { OP_Callback,     1,  0,  0},
           };
           if( pIdx->tnum==0 ) continue;
