@@ -543,10 +543,15 @@ void sqlite3VdbePrintSql(Vdbe *p){
 ** as allocating stack space and initializing the program counter.
 ** After the VDBE has be prepped, it can be executed by one or more
 ** calls to sqlite3VdbeExec().  
+**
+** This is the only way to move a VDBE from VDBE_MAGIC_INIT to
+** VDBE_MAGIC_RUN.
 */
 void sqlite3VdbeMakeReady(
   Vdbe *p,                       /* The VDBE */
   int nVar,                      /* Number of '?' see in the SQL statement */
+  int nMem,                      /* Number of memory cells to allocate */
+  int nCursor,                   /* Number of cursors to allocate */
   int isExplain                  /* True if the EXPLAIN keywords is present */
 ){
   int n;
@@ -568,17 +573,31 @@ void sqlite3VdbeMakeReady(
   ** Allocation all the stack space we will ever need.
   */
   if( p->aStack==0 ){
-    p->nVar = nVar;
     assert( nVar>=0 );
     n = isExplain ? 10 : p->nOp;
     p->aStack = sqliteMalloc(
       n*(sizeof(p->aStack[0])+sizeof(Mem*))          /* aStack, apArg */
-      + p->nVar*sizeof(Mem)                          /* apVar */
+      + nVar*sizeof(Mem)                             /* aVar */
+      + nVar*sizeof(char*)                           /* azVar */
+      + nMem*sizeof(Mem)                             /* aMem */
+      + nCursor*sizeof(Cursor*)                      /* apCsr */
     );
-    p->apArg = (Mem **)&p->aStack[n];
-    p->apVar = (Mem *)&p->apArg[n];
-    for(n=0; n<p->nVar; n++){
-      p->apVar[n].flags = MEM_Null;
+    if( !sqlite3_malloc_failed ){
+      p->apArg = (Mem **)&p->aStack[n];
+      p->aVar = (Mem *)&p->apArg[n];
+      p->azVar = (char**)&p->aVar[nVar];
+      p->okVar = 0;
+      p->nVar = nVar;
+      p->aMem = (Mem*)&p->azVar[nVar];
+      p->nMem = nMem;
+      p->apCsr = (Cursor**)&p->aMem[nMem];
+      p->nCursor = nCursor;
+      for(n=0; n<nVar; n++){
+        p->aVar[n].flags = MEM_Null;
+      }
+      for(n=0; n<nMem; n++){
+        p->aMem[n].flags = MEM_Null;
+      }
     }
   }
 
@@ -797,12 +816,11 @@ void sqlite3VdbeFreeCursor(Cursor *pCx){
 */
 static void closeAllCursors(Vdbe *p){
   int i;
+  if( p->apCsr==0 ) return;
   for(i=0; i<p->nCursor; i++){
     sqlite3VdbeFreeCursor(p->apCsr[i]);
+    p->apCsr[i] = 0;
   }
-  sqliteFree(p->apCsr);
-  p->apCsr = 0;
-  p->nCursor = 0;
 }
 
 /*
@@ -823,33 +841,14 @@ static void Cleanup(Vdbe *p){
     p->pTos = pTos;
   }
   closeAllCursors(p);
-  if( p->aMem ){
-    for(i=0; i<p->nMem; i++){
-      sqlite3VdbeMemRelease(&p->aMem[i]);
-    }
+  for(i=0; i<p->nMem; i++){
+    sqlite3VdbeMemRelease(&p->aMem[i]);
   }
-  sqliteFree(p->aMem);
-  p->aMem = 0;
-  p->nMem = 0;
   if( p->pList ){
     sqlite3VdbeKeylistFree(p->pList);
     p->pList = 0;
   }
   sqlite3VdbeSorterReset(p);
-  if( p->pFile ){
-    if( p->pFile!=stdin ) fclose(p->pFile);
-    p->pFile = 0;
-  }
-  if( p->azField ){
-    sqliteFree(p->azField);
-    p->azField = 0;
-  }
-  p->nField = 0;
-  if( p->zLine ){
-    sqliteFree(p->zLine);
-    p->zLine = 0;
-  }
-  p->nLineAlloc = 0;
   sqlite3VdbeAggReset(0, &p->agg, 0);
   if( p->keylistStack ){
     int ii;
@@ -942,7 +941,11 @@ static int vdbeCommit(sqlite *db){
 
   /* If there are any write-transactions at all, invoke the commit hook */
   if( needXcommit && db->xCommitCallback ){
-    if( db->xCommitCallback(db->pCommitArg) ){
+    int rc;
+    sqlite3SafetyOff(db);
+    rc = db->xCommitCallback(db->pCommitArg);
+    sqlite3SafetyOn(db);
+    if( rc ){
       return SQLITE_CONSTRAINT;
     }
   }
@@ -1128,6 +1131,8 @@ static void abortOtherActiveVdbes(Vdbe *pVdbe){
 ** This routine checks that the sqlite3.activeVdbeCnt count variable
 ** matches the number of vdbe's in the list sqlite3.pVdbe that are
 ** currently active. An assertion fails if the two counts do not match.
+** This is an internal self-check only - it is not an essential processing
+** step.
 **
 ** This is a no-op if NDEBUG is defined.
 */
@@ -1135,15 +1140,13 @@ static void abortOtherActiveVdbes(Vdbe *pVdbe){
 static void checkActiveVdbeCnt(sqlite *db){
   Vdbe *p;
   int cnt = 0;
-
   p = db->pVdbe;
   while( p ){
-    if( (p->magic==VDBE_MAGIC_RUN && p->pc>=0) || p->magic==VDBE_MAGIC_HALT ){
+    if( p->magic==VDBE_MAGIC_RUN && p->pc>=0 ){
       cnt++;
     }
     p = p->pNext;
   }
-
   assert( cnt==db->activeVdbeCnt );
 }
 #else
@@ -1151,60 +1154,41 @@ static void checkActiveVdbeCnt(sqlite *db){
 #endif
 
 /*
-** Clean up a VDBE after execution but do not delete the VDBE just yet.
-** Write any error messages into *pzErrMsg.  Return the result code.
+** This routine is called the when a VDBE tries to halt.  If the VDBE
+** has made changes and is in autocommit mode, then commit those
+** changes.  If a rollback is needed, then do the rollback.
 **
-** After this routine is run, the VDBE should be ready to be executed
-** again.
+** This routine is the only way to move the state of a VM from
+** SQLITE_MAGIC_RUN to SQLITE_MAGIC_HALT.
+**
+** Return an error code.  If the commit could not complete because of
+** lock contention, return SQLITE_BUSY.  If SQLITE_BUSY is returned, it
+** means the close did not happen and needs to be repeated.
 */
-int sqlite3VdbeReset(Vdbe *p){
+int sqlite3VdbeHalt(Vdbe *p){
   sqlite *db = p->db;
   int i;
   int (*xFunc)(Btree *pBt) = 0;  /* Function to call on each btree backend */
 
-  if( p->magic!=VDBE_MAGIC_RUN && p->magic!=VDBE_MAGIC_HALT ){
-    sqlite3Error(p->db, SQLITE_MISUSE, 0 ,0);
-    return SQLITE_MISUSE;
+  if( p->magic!=VDBE_MAGIC_RUN ){
+    /* Already halted.  Nothing to do. */
+    assert( p->magic==VDBE_MAGIC_HALT );
+    return SQLITE_OK;
   }
-  if( p->zErrMsg ){
-    sqlite3Error(p->db, p->rc, "%s", p->zErrMsg, 0);
-    sqliteFree(p->zErrMsg);
-    p->zErrMsg = 0;
-  }else if( p->rc ){
-    sqlite3Error(p->db, p->rc, 0);
-  }else{
-    sqlite3Error(p->db, SQLITE_OK, 0);
-  }
-  Cleanup(p);
-
-  /* What is done now depends on the exit status of the vdbe, the value of
-  ** the sqlite.autoCommit flag and whether or not there are any other
-  ** queries in progress. A transaction or statement transaction may need
-  ** to be committed or rolled back on each open database file.
-  */
+  closeAllCursors(p);
   checkActiveVdbeCnt(db);
   if( db->autoCommit && db->activeVdbeCnt==1 ){
     if( p->rc==SQLITE_OK || p->errorAction==OE_Fail ){
       /* The auto-commit flag is true, there are no other active queries
       ** using this handle and the vdbe program was successful or hit an
-      ** 'OR FAIL' constraint. This means a commit is required, which is
-      ** handled a little differently from the other options.
+      ** 'OR FAIL' constraint. This means a commit is required.
       */
-      p->rc = vdbeCommit(db);
-      if( p->rc!=SQLITE_OK ){
-        sqlite3Error(p->db, p->rc, 0);
-        if( p->rc==SQLITE_BUSY && p->autoCommitOn ){
-          /* If we just now have turned autocommit on (meaning we just have
-          ** finished executing a COMMIT command) but the commit fails due
-          ** to lock contention, autocommit back off.  This gives the user
-          ** the opportunity to try again after the lock that was preventing
-          ** the commit has cleared. */
-          db->autoCommit = 0;
-        }else{
-          /* If the command just executed was not a COMMIT command, then
-          ** rollback whatever the results of that command were */
-          xFunc = sqlite3BtreeRollback;
-        }
+      int rc = vdbeCommit(db);
+      if( rc==SQLITE_BUSY ){
+        return SQLITE_BUSY;
+      }else if( rc!=SQLITE_OK ){
+        p->rc = rc;
+        xFunc = sqlite3BtreeRollback;
       }
     }else{
       xFunc = sqlite3BtreeRollback;
@@ -1220,7 +1204,6 @@ int sqlite3VdbeReset(Vdbe *p){
       abortOtherActiveVdbes(p);
     }
   }
-  p->autoCommitOn = 0;
 
   /* If xFunc is not NULL, then it is one of sqlite3BtreeRollback,
   ** sqlite3BtreeRollbackStmt or sqlite3BtreeCommitStmt. Call it once on
@@ -1246,16 +1229,65 @@ int sqlite3VdbeReset(Vdbe *p){
     p->nChange = 0;
   }
 
+  /* Rollback or commit any schema changes that occurred. */
   if( p->rc!=SQLITE_OK ){
     sqlite3RollbackInternalChanges(db);
   }else if( db->flags & SQLITE_InternChanges ){
     sqlite3CommitInternalChanges(db);
   }
 
-  if( (p->magic==VDBE_MAGIC_RUN && p->pc>=0) || p->magic==VDBE_MAGIC_HALT ){
+  /* We have successfully halted and closed the VM.  Record this fact. */
+  if( p->pc>=0 ){
     db->activeVdbeCnt--;
   }
+  p->magic = VDBE_MAGIC_HALT;
+  checkActiveVdbeCnt(db);
 
+  return SQLITE_OK;
+}
+
+/*
+** Clean up a VDBE after execution but do not delete the VDBE just yet.
+** Write any error messages into *pzErrMsg.  Return the result code.
+**
+** After this routine is run, the VDBE should be ready to be executed
+** again.
+**
+** To look at it another way, this routine resets the state of the
+** virtual machine from VDBE_MAGIC_RUN or VDBE_MAGIC_HALT back to
+** VDBE_MAGIC_INIT.
+*/
+int sqlite3VdbeReset(Vdbe *p){
+  if( p->magic!=VDBE_MAGIC_RUN && p->magic!=VDBE_MAGIC_HALT ){
+    sqlite3Error(p->db, SQLITE_MISUSE, 0 ,0);
+    return SQLITE_MISUSE;
+  }
+
+  /* If the VM did not run to completion or if it encountered an
+  ** error, then it might not have been halted properly.  So halt
+  ** it now.
+  */
+  sqlite3VdbeHalt(p);
+
+  /* Transfer the error code and error message from the VDBE into the
+  ** main database structure.
+  */
+  if( p->zErrMsg ){
+    sqlite3Error(p->db, p->rc, "%s", p->zErrMsg, 0);
+    sqliteFree(p->zErrMsg);
+    p->zErrMsg = 0;
+  }else if( p->rc ){
+    sqlite3Error(p->db, p->rc, 0);
+  }else{
+    sqlite3Error(p->db, SQLITE_OK, 0);
+  }
+
+  /* Reclaim all memory used by the VDBE
+  */
+  Cleanup(p);
+
+  /* Save profiling information from this VDBE run.
+  */
   assert( p->pTos<&p->aStack[p->pc<0?0:p->pc] || sqlite3_malloc_failed==1 );
 #ifdef VDBE_PROFILE
   {
@@ -1283,7 +1315,7 @@ int sqlite3VdbeReset(Vdbe *p){
   p->aborted = 0;
   return p->rc;
 }
-
+ 
 /*
 ** Clean up and delete a VDBE after execution.  Return an integer which is
 ** the result code.  Write any error message text into *pzErrMsg.
@@ -1357,7 +1389,7 @@ void sqlite3VdbeDelete(Vdbe *p){
     }
   }
   for(i=0; i<p->nVar; i++){
-    sqlite3VdbeMemRelease(&p->apVar[i]);
+    sqlite3VdbeMemRelease(&p->aVar[i]);
   }
   sqliteFree(p->aOp);
   sqliteFree(p->aLabel);
